%% Script to perform a rolling-window analysis of the VEC model for different dwelling types.
%----Author: Guru Raghav Raman----
%Run after executing the script "Load_data.m".

%% Calculate class-wise aggregate demand.
demand_classwise=zeros(48,213,6); %Initialization, classwise aggregate demand

for class_to_study=1:6
    list_of_consumers=find(class_final==class_to_study)';    
    for consumer=list_of_consumers
        demand_classwise(:,:,class_to_study) = demand_classwise(:,:,class_to_study)+ (2*demand_data(:,:,consumer)); %Multiplication by 2 to convert kWh per half-hour interval to kW values
    end
end

peak_demand_classwise=max(demand_classwise,[],1);

%% Vector Error Correction (VEC) models
color_scheme={'#8c510a','#bf812d','#dfc27d','#80cdc1','#35978f','#01665e'};
    
window_size=7*10; %Each window is 10 weeks in length
first_date=84; 
dates_to_check=first_date:2:142;

for which_class_to_study=1:6
    
    output_series=[]; %Initialization.
    
    %These are used to plot how much of the training period is before the
    %lockdown period.
    day_of_lockdown=datetime(2020,04,7,0,0,0); %7 April 2020
    percentage_before_lockdown=[]; %Initialization

    for start_date=dates_to_check
        disp(['Start date: ', num2str(start_date)]);
        target_dates=start_date+(0:(window_size-1));

        %Prerequisites
        %Difference the time-series data to make them stationairy
        input_1=diff(peak_demand_classwise(1,target_dates,which_class_to_study)');
        input_2=diff(case_data(target_dates,1));
        input_3=diff(case_data(target_dates,2));
        input_4=mean(weather_1,1)';
        input_5=mean(weather_2,1)';
        input_4=diff(input_4(target_dates));
        input_5=diff(input_5(target_dates));
        input_set=[input_1 input_2 input_3 input_4 input_5];

        %This is the set of original inputs, used for co-integration tests only.
        weather_term_1=mean(weather_1,1)';
        weather_term_2=mean(weather_2,1)';
        input_original=[peak_total_demand(target_dates)' case_data(target_dates,1) case_data(target_dates,2)  weather_term_1(target_dates) weather_term_2(target_dates)];

        %Check for stationarity of the different time series data using Augmented
        %Dickey-Fuller test.
        [h1,p_value1]=adftest(input_1);
        [h2,p_value2]=adftest(input_2);
        [h3,p_value3]=adftest(input_3);
        [h4,p_value4]=adftest(input_4);
        [h5,p_value5]=adftest(input_5);

        if (p_value1>0.05 || p_value2>0.05 || p_value3>0.05 || p_value4>0.05 || p_value5>0.05 || h1*h2*h3*h4*h5==0)
            disp('INPUT NOT STATIONARY');
        end

        %Johansen cointegration test
        [h,pValue,stat,cValue,mles] = jcitest(input_original); 

        %Number of cointegrating relations found from the Johansen cointegration test.
        if h.r0==0
            rank_vecm=0;
        elseif h.r1==0
            rank_vecm=1;
        elseif h.r2==0
            rank_vecm=2;
        elseif h.r3==0
            rank_vecm=3;
        elseif h.r4==0
            rank_vecm=4;    
        end

        %Find out which parameters work the best and fit vector autoregression.
        numseries = 5; %Number of series in the input
        numlags = (1:7)'; %Which model size (lags) to test for the best one.
        nummdls = numel(numlags);
        T=size(input_1,1);

        % Partition time base.
        maxp = max(numlags);
        idxpre = 1:maxp;
        idxest = (maxp + 1):T;

        % Preallocation
        EstMdl(nummdls) = vecm(numseries,rank_vecm,0);
        aic = zeros(nummdls,1);

        % Fit models to data.
        Y0 = input_set(idxpre,:); 
        Y = input_set(idxest,:);
        for j = 1:numel(numlags)
            Mdl = vecm(numseries,rank_vecm,(numlags(j)-1));
            Mdl.SeriesNames = {{'peakdemand'},{'newcases'},{'recoveredcases'},{'weather1'},{'weather2'}};
            EstMdl(j) = estimate(Mdl,Y,'Y0',Y0);
            results = summarize(EstMdl(j));
            aic(j) = results.AIC;
        end

        [~,bestidx] = min(aic);

        %Best model
        BestMdl = EstMdl(bestidx);

        %Forecast Error Variance Decomposition (EVD)
        time_steps=100; %No. of timesteps to forecast error variance decomposition.
        Decomposition = fevd(BestMdl,'NumObs',time_steps);

        %Save the result
        output_series=[output_series Decomposition(100,2,1)];
        percentage_before_lockdown=[percentage_before_lockdown (100*nnz(t(target_dates)<day_of_lockdown)/size(target_dates,2))];
    end

    % Plot results
    figure(11)
    subplot(2,1,2)
    plot(t(dates_to_check),100*output_series,'-','LineWidth',3,'Color',color_scheme{which_class_to_study});
    hold on
    xlim([t(dates_to_check(1)) t(dates_to_check(end))])
    ylim([0 110])
end

%% Plot results
figure(11)
subplot(2,1,2)
legend({'1-Room / 2-Room HDB','3-Room HDB','4-Room HDB','5-Room and Executive HDB','Private apartment and condominium','Landed property'});

%Plot how much percent of the window is before lockdown.
subplot(2,1,1)
plot_color='#5D8AA8';
plot(t(dates_to_check),percentage_before_lockdown,'-','LineWidth',2,'Color',plot_color);
hold on
area(t(dates_to_check),percentage_before_lockdown,'LineStyle','None','FaceAlpha',0.3,'FaceColor',plot_color,'EdgeColor',plot_color);
xlim([t(dates_to_check(1)) t(dates_to_check(end))])
set(gca,'xticklabel',{[]});
box off